@@ -3,10 +3,10 @@ use warnings;
 use strict;
 
 # Copyright 2004 Randy Smith
-# $Id: ExtHandler.pm,v 1.42 2006/01/04 21:57:48 perlstalker Exp $
+# $Id: ExtHandler.pm,v 1.44 2006/09/28 17:10:51 perlstalker Exp $
 
-our $REVISION = (split (' ', '$Revision: 1.42 $'))[1];
-our $VERSION = "0.3.0";
+our $REVISION = (split (' ', '$Revision: 1.44 $'))[1];
+our $VERSION = "0.3.1";
 
 use lib qw(..);
 use Getopt::Long;
@@ -313,8 +313,14 @@ sub get_options
     my $self = shift;
     my $keyword = shift;
     my $action = shift;
+    
+    my $real_act = $action;
+    if (not defined $self->{keywords}{$keyword}{$action}
+        and defined $self->{keywords}{$keyword}{'*'}) {
+        $real_act = '*';
+    }
 
-    return sort keys %{ $self->{keywords}{$keyword}{$action}{options}};
+    return sort keys %{ $self->{keywords}{$keyword}{$real_act}{options}};
 }
 
 # Return unsorted list of VUser::Meta objects;
@@ -361,6 +367,7 @@ sub load_extensions
     my %cfg = @_;
 
     $self->{'_loaded'} = {};
+    $self->{'_loadorder'} = [];
 
     $self->load_extension('CORE');
     my $exts = $cfg{ vuser }{ extensions };
@@ -408,6 +415,8 @@ sub load_extension
     }
        
     $log->log(LOG_INFO, "Loading extension: $ext");
+    $self->{'_loaded'}{$ext} = 1;
+    push (@{$self->{'_loadorder'}}, $ext);
     &{$pm.'::init'}($self, %cfg);
 }
 
@@ -416,7 +425,8 @@ sub unload_extensions
     my $self = shift;
     my %cfg = @_;
 
-    foreach my $ext (keys %{ $self->{'_loaded'} }) {
+    # foreach my $ext (keys %{ $self->{'_loaded'} }) {
+    foreach my $ext (reverse (@{$self->{'_loadorder'}})) {
 	eval { $self->unload_extension($ext, %cfg); };
 	warn "Unable to unload $ext: $@\n" if $@;
     }
@@ -431,7 +441,10 @@ sub unload_extension
     my $pm = 'VUser::'.$ext;
 
     no strict ('refs');
-    &{$pm.'::unload'}($self, %cfg);
+    $log->log(LOG_INFO, "Unloading extension: $ext");
+    if (UNIVERSAL::can($pm, 'unload')) {
+	&{$pm.'::unload'}($self, %cfg);
+    }
 }
 
 sub run_tasks
